package com.ylx.web.core.config;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import com.ylx.common.config.RuoYiConfig;
import io.swagger.models.auth.In;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.contexts.SecurityContext;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2WebMvc;

/**
 * Swagger2的接口配置
 *
 * @author ylx
 */
@EnableSwagger2WebMvc
@Configuration
public class SwaggerConfig {
    //** 系统基础配置 *//*
    @Autowired
    private RuoYiConfig ruoyiConfig;

    //** 是否开启swagger *//*
    @Value("${swagger.enabled}")
    private boolean enabled;

    //** 设置请求的统一前缀 *//*
    @Value("${swagger.pathMapping}")
    private String pathMapping;


    @Bean
    public Docket createRestApi()
    {
        return new Docket(DocumentationType.SWAGGER_2)
                // 是否启用Swagger
                .enable(enabled)
                // 用来创建该API的基本信息，展示在文档的页面中（自定义展示的信息）
                .apiInfo(apiInfo())
                // 设置哪些接口暴露给Swagger展示
                .select()
                // 扫描所有有注解的api，用这种方式更灵活
                //.apis(RequestHandlerSelectors.withMethodAnnotation(ApiOperation.class))
                // 扫描指定包中的swagger注解
                // .apis(RequestHandlerSelectors.basePackage("com.ylx.project.tool.swagger"))
                // 扫描所有
                .apis(RequestHandlerSelectors.any())
                .paths(PathSelectors.any())
                .build()
                // 设置安全模式，swagger可以设置访问token *//*
                .securitySchemes(securitySchemes())
                .securityContexts(securityContexts());
//                .pathMapping(pathMapping);
    }


    private List<SecurityScheme> securitySchemes()
    {
        List<SecurityScheme> apiKeyList = new ArrayList<SecurityScheme>();
        apiKeyList.add(new ApiKey("Authorization", "Authorization", In.HEADER.toValue()));
        return apiKeyList;
    }


    private List<SecurityContext> securityContexts()
    {
        List<SecurityContext> securityContexts = new ArrayList<>();
        securityContexts.add(
                SecurityContext.builder()
                        .securityReferences(defaultAuth())
                        .operationSelector(o -> o.requestMappingPattern().matches("/.*"))
                        .build());
        return securityContexts;
    }


    private List<SecurityReference> defaultAuth()
    {
        AuthorizationScope authorizationScope = new AuthorizationScope("global", "accessEverything");
        AuthorizationScope[] authorizationScopes = new AuthorizationScope[1];
        authorizationScopes[0] = authorizationScope;
        List<SecurityReference> securityReferences = new ArrayList<>();
        securityReferences.add(new SecurityReference("Authorization", authorizationScopes));
        return securityReferences;
    }


    private ApiInfo apiInfo()
    {
        // 用ApiInfoBuilder进行定制
        return new ApiInfoBuilder()
                // 设置标题
                .title("夜来香系统_接口文档")
                // 描述
                .description("描述：体包括XXX,XXX模块...")
                // 作者信息
                .contact(new Contact(ruoyiConfig.getName(), null, null))
                // 版本
                .version("版本号:" + ruoyiConfig.getVersion())
                .build();
    }

//=======================================================================================


//    /**
//     * @version 1.0
//     * @author jinhaoxun
//     * @date 2019-08-09
//     * @description Swagger配置
//     */
//
//
//    /**
//     * 定义分隔符
//     */
//    private static final String SPLITOR = ";";
//
//    /**
//     * @return Docket
//     * @author jinhaoxun
//     * @description 配置token，以及设置扫描包的路径
//     */
//    @Bean("createRestApi1")
//    public Docket createRestApi1() {
//        //设置header里面的token
//        ParameterBuilder tokenPar = new ParameterBuilder();
//        List<Parameter> pars = new ArrayList<>();
//        tokenPar.name("token").description("token令牌").modelRef(new ModelRef("string")).parameterType("header").required(false).build();
//        pars.add(tokenPar.build());
//
//        return new Docket(DocumentationType.SWAGGER_2)
//                .globalOperationParameters(pars)
//                .apiInfo(apiInfo())
//                .groupName("1.0.0 版本")
//                .select()
//                //此处添加需要扫描接口的包路径
//                .apis(basePackage("com.ylx.web" + SPLITOR
//                        + "com.ylx.massage" + SPLITOR))
//                .paths(PathSelectors.any())
//                .build();
//    }
//
//    /**
//     * @return Docket
//     * @author jinhaoxun
//     * @description 配置token，以及设置扫描包的路径
//     */
//    @Bean("createRestApi2")
//    public Docket createRestApi2() {
//        //设置header里面的token
//        ParameterBuilder tokenPar = new ParameterBuilder();
//        List<Parameter> pars = new ArrayList<>();
//        tokenPar.name("token").description("token").modelRef(new ModelRef("string")).parameterType("header").required(false).build();
//        pars.add(tokenPar.build());
//
//        return new Docket(DocumentationType.SWAGGER_2)
//                .globalOperationParameters(pars)
//                .apiInfo(apiInfo())
//                .groupName("2.0.0 版本")
//                .select()
//                //此处添加需要扫描接口的包路径
//                .apis(basePackage("com.ylx.web" + SPLITOR
//                        + "com.ylx.massage" + SPLITOR))
//                .paths(PathSelectors.any())
//                .build();
//    }
//
//    /**
//     * @return Docket
//     * @author jinhaoxun
//     * @description 配置Swagger2页面显示内容
//     */
//    private ApiInfo apiInfo() {
//        return new ApiInfoBuilder()
//                .title("Swagger 接口文档")
//                .description("Swagger 接口文档")
//                .version("1.0.0")
//                .termsOfServiceUrl("http://localhost:8080/swagger-ui.html")
//                .contact(new Contact(ruoyiConfig.getName(), null, null))
//                .build();
//    }
//
//    /**
//     * @param basePackage 所有包路径
//     * @return Predicate<RequestHandler>
//     * @author jinhaoxun
//     * @description 重写basePackage方法，使能够实现多包访问
//     */
//    public static Predicate<RequestHandler> basePackage(final String basePackage) {
//        return input -> declaringClass(input).map(handlerPackage(basePackage)::apply).orElse(true);
//    }
//
//    /**
//     * @param basePackage 所有包路径
//     * @return Function<Class < ?>, Boolean>
//     * @author jinhaoxun
//     * @description 重写basePackage方法，使能够实现多包访问
//     */
//    private static Function<Class<?>, Boolean> handlerPackage(final String basePackage) {
//        return input -> {
//            // 循环判断匹配
//            for (String strPackage : basePackage.split(SPLITOR)) {
//                assert input != null;
//                boolean isMatch = input.getPackage().getName().startsWith(strPackage);
//                if (isMatch) {
//                    return true;
//                }
//            }
//            return false;
//        };
//    }
//
//    /**
//     * @param input
//     * @return Optional<? extends Class < ?>>
//     * @author jinhaoxun
//     * @description 重写basePackage方法，使能够实现多包访问
//     */
//    private static Optional<Class<?>> declaringClass(RequestHandler input) {
//        return Optional.ofNullable(input.declaringClass());
//    }


}
