package com.ylx.massage.domain;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import com.ylx.common.annotation.Excel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import lombok.experimental.Accessors;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;

/**
 * <p>
 * 认领表
 * </p>
 *
 * @author YJR
 * @since 2023-12-30
 */
@Getter
@Setter
@Accessors(chain = true)
@TableName(value = "t_order",autoResultMap = true)
@ApiModel(value = "TOrder", description = "订单认领表")
public class TOrder implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId("c_id")
    @ApiModelProperty("cId")
    private String cId;

    /**
     * 技师id
     */
    @TableField("c_js_id")
    @ApiModelProperty("技师id")
    private String cJsId;

    @TableField("old_js_id")
    @ApiModelProperty("新技师id")
    private String oldJsId;

    @Excel(name = "订单号")
    @TableField("order_no")
    @ApiModelProperty("订单号")
    private String orderNo;

    @TableField("parent_no")
    @ApiModelProperty("父订单号")
    private String parentNo;

    @TableField("timeout_cause")
    @ApiModelProperty("超时原因")
    private String timeoutCause;


    @TableField("tape")
    @ApiModelProperty("音频")
    private String tape;

    @TableField("fare")
    @ApiModelProperty("车费")
    private BigDecimal fare;

    @TableField("distance")
    @ApiModelProperty("距离")
    private BigDecimal distance;

    @TableField("pay_type")
    @ApiModelProperty("支付类型:1微信支付 2:余额支付")
    private Integer payType;

    @TableField("preferential")
    @ApiModelProperty("优惠金额")
    private BigDecimal preferential;

    @TableField("price_difference")
    @ApiModelProperty("差价")
    private BigDecimal priceDifference;

    @TableField("order_type")
    @ApiModelProperty("订单类型:0普通订单,1加钟订单,2升级订单")
    private Integer orderType;
    /**
     * 用户openid
     */
    @TableField("c_open_id")
    @ApiModelProperty("用户openid")
    private String cOpenId;

    //经度
    @ApiModelProperty("经度")
    private Double longitude;
    //纬度
    @ApiModelProperty("纬度")
    private Double latitude;

    //经度
    @ApiModelProperty("出发经度")
    private BigDecimal departLongitude;
    //纬度
    @ApiModelProperty("出发纬度")
    private BigDecimal departLatitude;

    //经度
    @ApiModelProperty("到达经度")
    private BigDecimal arrivalLongitude;
    //纬度
    @ApiModelProperty("到达纬度")
    private BigDecimal arrivalLatitude;

    @ApiModelProperty("到达照片")
    private String arrivalPhoto;

    @ApiModelProperty("出发时间")
    private Date departTime;

    @ApiModelProperty("支付时间")
    private Date payTime;

    //详细地址
    @ApiModelProperty("详细地址")
    private String address;

    @ApiModelProperty("地址name")
    private String name;

    @ApiModelProperty("接单时间")
    private LocalDateTime acceptanceTime;

    @ApiModelProperty("到达时间")
    private LocalDateTime reachTime;

    @ApiModelProperty("开始服务时间")
    private LocalDateTime startTime;

    @ApiModelProperty("结束时间")
    private LocalDateTime endTime;

    @ApiModelProperty("拒接原因")
    private String reasonRefusal;

    /**
     * 领取优惠卷id
     */
    @TableField("coupon_receive_id")
    @ApiModelProperty("领取优惠卷id")
    private String couponReceiveId;

    /**
     * 订单明细
     */
    @TableField(value = "c_goods",typeHandler = FastjsonTypeHandler.class)
    @ApiModelProperty("订单明细")
    private JSONArray cGoods;

    /**
     * 订单金额
     */
    @TableField("d_total_money")
    @ApiModelProperty("订单金额")
    private BigDecimal dTotalMoney;

    /**
     * 总价
     */
    @Excel(name = "订单金额")
    @TableField("total_price")
    @ApiModelProperty("总价")
    private BigDecimal totalPrice;

    /**
     * 订单状态	订单状态 待接单 0 已接单 1 已到达 2 服务中 3 待评价（已完成） 4 待付款 -1 已取消 -2 已拒绝 -3 已完成（已评价）5
     */
    @TableField("n_status")
    @ApiModelProperty("订单状态 待接单0  已接单1  已到达2  服务中3  待评价（已完成4  待付款-1  已取消-2  已拒绝-3  已完成（已评价）5   已出发6")
    private Integer nStatus;

    /**
     * 地址
     */
    @TableField("c_address")
    @ApiModelProperty("地址")
    private String cAddress;

    @ApiModelProperty("备注地址")
    private String atlasAdd;

    /**
     * 姓名
     */
    @Excel(name = "用户姓名")
    @TableField("c_name")
    @ApiModelProperty("姓名")
    private String cName;

    /**
     * 电话
     */
    @Excel(name = "用户电话")
    @TableField("c_phone")
    @ApiModelProperty("电话")
    private String cPhone;

    /**
     * 备注
     */
    @TableField("c_note")
    @ApiModelProperty("备注")
    private String cNote;

    /**
     * 时间
     */
    @TableField("c_time")
    @ApiModelProperty("支付截止时间")
    private String cTime;

    /**
     * 推荐者id
     */
    @TableField("c_tj_open_id")
    @ApiModelProperty("推荐者id")
    private String cTjOpenId;

    @TableField("dept_id")
    @ApiModelProperty("部门Id")
    private String deptId;

    @TableField("dept_name")
    @ApiModelProperty("部门名称")
    private String deptName;

    /**
     * 创建时间
     */
    @Excel(name = "时间")
    @TableField("dt_create_time")
    @ApiModelProperty("创建时间")
    private LocalDateTime dtCreateTime;

    @ApiModelProperty("是否删除0否1是")
    @TableLogic
    private Integer isDelete;

    @TableField(exist = false)
    @ApiModelProperty("技师信息")
    private TJs js;

    @TableField(exist = false)
    @ApiModelProperty("剩余支付时间(秒)")
    private Long remainingTime;

    @TableField(exist = false)
    @ApiModelProperty("技师昵称")
    private String cNickName;

    @Excel(name = "技师姓名")
    @TableField(exist = false)
    @ApiModelProperty("技师姓名")
    private String jsName;

    @Excel(name = "技师电话")
    @TableField(exist = false)
    @ApiModelProperty("技师手机")
    private String jsPhone;

    @TableField(exist = false)
    @ApiModelProperty("用户信息")
    private TWxUser wxUser;

    @TableField(exist = false)
    @Excel(name = "订单状态")
    @ApiModelProperty("订单状态名称")
    private String statusName;

    public String getcId() {
        return cId;
    }

    public void setcId(String cId) {
        this.cId = cId;
    }

    public String getcJsId() {
        return cJsId;
    }

    public void setcJsId(String cJsId) {
        this.cJsId = cJsId;
    }

    public String getcOpenId() {
        return cOpenId;
    }

    public void setDepartLongitude(BigDecimal departLongitude)
    {
        this.departLongitude = departLongitude;
    }

    public BigDecimal getDepartLongitude()
    {
        return departLongitude;
    }

    public void setArrivalLongitude(BigDecimal arrivalLongitude)
    {
        this.arrivalLongitude = arrivalLongitude;
    }

    public BigDecimal getArrivalLongitude()
    {
        return arrivalLongitude;
    }

    public void setArrivalPhoto(String arrivalPhoto)
    {
        this.arrivalPhoto = arrivalPhoto;
    }

    public String getArrivalPhoto()
    {
        return arrivalPhoto;
    }

    public void setDepartTime(Date departTime)
    {
        this.departTime = departTime;
    }

    public Date getDepartTime()
    {
        return departTime;
    }

    public void setDepartLatitude(BigDecimal departLatitude)
    {
        this.departLatitude = departLatitude;
    }

    public BigDecimal getDepartLatitude()
    {
        return departLatitude;
    }

    public void setArrivalLatitude(BigDecimal arrivalLatitude)
    {
        this.arrivalLatitude = arrivalLatitude;
    }

    public BigDecimal getArrivalLatitude()
    {
        return arrivalLatitude;
    }

    public void setcOpenId(String cOpenId) {
        this.cOpenId = cOpenId;
    }

    public JSONArray getcGoods() {
        return cGoods;
    }

    public void setcGoods(JSONArray cGoods) {
        this.cGoods = cGoods;
    }

    public BigDecimal getdTotalMoney() {
        return dTotalMoney;
    }

    public void setdTotalMoney(BigDecimal dTotalMoney) {
        this.dTotalMoney = dTotalMoney;
    }

    public Integer getnStatus() {
        return nStatus;
    }

    public void setnStatus(Integer nStatus) {
        this.nStatus = nStatus;
    }

    public String getcAddress() {
        return cAddress;
    }

    public void setcAddress(String cAddress) {
        this.cAddress = cAddress;
    }

    public String getcName() {
        return cName;
    }

    public void setcName(String cName) {
        this.cName = cName;
    }

    public String getcPhone() {
        return cPhone;
    }

    public void setcPhone(String cPhone) {
        this.cPhone = cPhone;
    }

    public String getcNote() {
        return cNote;
    }

    public void setcNote(String cNote) {
        this.cNote = cNote;
    }

    public String getcTime() {
        return cTime;
    }

    public void setcTime(String cTime) {
        this.cTime = cTime;
    }

    public String getcTjOpenId() {
        return cTjOpenId;
    }

    public void setcTjOpenId(String cTjOpenId) {
        this.cTjOpenId = cTjOpenId;
    }

    public LocalDateTime getDtCreateTime() {
        return dtCreateTime;
    }

    public void setDtCreateTime(LocalDateTime dtCreateTime) {
        this.dtCreateTime = dtCreateTime;
    }

    public TJs getJs() {
        return js;
    }

    public void setJs(TJs js) {
        this.js = js;
    }

    public TWxUser getWxUser() {
        return wxUser;
    }

    public void setWxUser(TWxUser wxUser) {
        this.wxUser = wxUser;
    }
}
