package com.ylx.massage.service.impl;

import cn.hutool.crypto.digest.DigestUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ylx.common.config.RuoYiConfig;
import com.ylx.common.core.domain.AjaxResult;
import com.ylx.common.utils.file.FileUploadUtils;
import com.ylx.common.utils.file.FileUtils;
import com.ylx.framework.config.ServerConfig;
import com.ylx.massage.domain.TbFile;
import com.ylx.massage.mapper.TbFileMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ylx.massage.service.TbFileService;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

/**
 * (TbFile)表服务实现类
 *
 * @author makejava
 * @since 2024-04-27 17:12:46
 */
@Service("tbFileService")
public class TbFileServiceImpl extends ServiceImpl<TbFileMapper, TbFile> implements TbFileService {

    @Autowired
    private ServerConfig serverConfig;
    @Override
    public TbFile getByMd5(String md5) {
        LambdaQueryWrapper<TbFile> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TbFile::getMd5,md5);
        return this.getOne(queryWrapper);
    }

    public String calculateMD5(MultipartFile file) throws IOException {
        byte[] bytes = file.getBytes();
        return DigestUtil.md5Hex(bytes);
    }
    @Override
    public AjaxResult uploadFile(MultipartFile file) {
        AjaxResult ajax = AjaxResult.success();
        try
        {
            String md5 = calculateMD5(file);

            TbFile dbFile = this.getByMd5(md5);
            if(null != dbFile){
                ajax.put("url", serverConfig.getUrl() + dbFile.getFileUrl());
                ajax.put("fileName", dbFile.getFileUrl());
                ajax.put("newFileName", FileUtils.getName(dbFile.getFileUrl()));
                ajax.put("originalFilename", dbFile.getFileName());
                return ajax;
            }

            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;
            ajax.put("url", url);
            ajax.put("fileName", fileName);
            ajax.put("newFileName", FileUtils.getName(fileName));
            ajax.put("originalFilename", file.getOriginalFilename());

            TbFile tbFile = new TbFile();
            tbFile.setMd5(md5);
            tbFile.setFileName(file.getOriginalFilename());
            tbFile.setFileUrl(fileName);
            this.save(tbFile);
            return ajax;
        }
        catch (Exception e)
        {
            return AjaxResult.error(e.getMessage());
        }
    }
}

